<?php
/**
* Plugin Name: Ochatbot and Ometrics Conversion Optimization Tools
* Plugin URI: http://www.ometrics.com/
* Version: 1.0.8
* Author: Ometrics, LLC
* Author URI: http://www.ometrics.com/
* WC optional - if used requires at least: 3.6.0
* WC tested up to: 4.9.1
* Description: Integrates the Ometrics widget script into your WordPress site
*  which allows all of the Ometrics tools (chatbot and other conversion
*  optimization tools) to be used on your site.
*  For WooCommerce sites, it also adds integration for conversion tracking,
*  abandoned cart processing and order status tracking.
*
* License: GPL3
*/

/*  Copyright 2020 Ometrics

*/

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Current plugin version.
 */
 if (!defined('WP_OMETRICS_VERSION'))
		define( 'WP_OMETRICS_VERSION', '1.0.8' );

register_uninstall_hook(__FILE__, 'ometricsUninstall');

function ometricsUninstall() {
	delete_option('ometrics_id');
	delete_option('ometrics_token');
	delete_option('wp-ometrics_welcome_dismissed_key');
}

/**
* WPOmetrics Class
*/
class WPOmetrics {
	/**
	* Constructor
	*/
	public function __construct() {

		// Plugin Details
		$this->plugin               = new stdClass;
		$this->plugin->name         = 'wp-ometrics'; // Plugin Folder
		$this->plugin->displayName  = 'Ochatbot and Ometrics Conversion Optimization Tools'; // Plugin Name
		$this->plugin->version      = WP_OMETRICS_VERSION;
		$this->plugin->folder       = plugin_dir_path( __FILE__ );
		$this->plugin->url          = plugin_dir_url( __FILE__ );
		$this->plugin->db_welcome_dismissed_key = 'wp-ometrics_welcome_dismissed_key';//?djb

		//internal api version
		$this->ometrics_api_version = 'v1';
		//woocommerce api version
		$this->wp_api_version = 'v2';

		//default hooks
		//register_activation_hook( __FILE__, array($this, 'wp-ometrics_activation') );
		//register_deactivation_hook( __FILE__, array($this, 'wp-ometrics_deactivation') );
		//register_uninstall_hook(__FILE__, array($this, 'ometricsUninstall'));

		// Hooks
		add_action( 'admin_init', array( $this, 'registerSettings' ) );
		add_action( 'admin_menu', array( $this, 'adminPanelsAndMetaBoxes' ) );
		add_action( 'admin_notices', array( $this, 'dashboardNotices' ) );
		add_action( 'wp_ajax_' . $this->plugin->name . '_dismiss_dashboard_notices', array( $this, 'dismissDashboardNotices' ) );

		// Frontend Hooks
		add_action( 'wp_head', array( $this, 'frontendHeader' ) );

		//conversion tracker
		add_action( 'woocommerce_thankyou', array( $this, 'checkoutComplete' ) );

		//custom API endpoints
		add_action( 'rest_api_init', array($this, 'registerRoutes')  );

		add_filter( 'woocommerce_rest_check_permissions', array( $this, 'allowRestCallForAuthenticatedPlugin' ), 90, 4 );
		add_action( 'wp_ajax_ometrics_submit', array($this, 'submitOmetricsSettings') );
		add_filter('plugins_api', array( $this, 'wp_ometrics_plugin_info'), 20, 3);
		add_filter('site_transient_update_plugins', array( $this, 'wp_ometrics_push_update') );
		add_action( 'upgrader_process_complete', array( $this, 'wp_ometrics_after_update'), 10, 2 );

	}

	function registerRoutes() {
		register_rest_route( 'wp-ometrics/v1', '/order/(?P<order_id>.+)', array(
			'methods' => 'GET',
			'permission_callback' => array( $this, 'ometricsPermissionCheck' ),
			'callback' => array($this, 'ochatbotOrderStatus'),
		) );

		register_rest_route( 'wp-ometrics/v1', '/products', array(
			'methods' => 'GET',
			'permission_callback' => array( $this, 'ometricsPermissionCheck' ),
			'callback' => array($this, 'ochatbotProductList'),
		) );

		register_rest_route( 'wp-ometrics/v1', '/products/(?P<product_id>[\d]+)/variations', array(
			'methods' => 'GET',
			'permission_callback' => array( $this, 'ometricsPermissionCheck' ),
			'callback' => array($this, 'ochatbotProductVariation'),
		) );

	}

	/*
	* $res empty at this step
	* $action 'plugin_information'
	* $args stdClass Object ( [slug] => woocommerce [is_ssl] => [fields] => Array ( [banners] => 1 [reviews] => 1 [downloaded] => [active_installs] => 1 ) [per_page] => 24 [locale] => en_US )
	*/
	function wp_ometrics_plugin_info( $res, $action, $args ){

		// do nothing if this is not about getting plugin information - testing comment out
		if( 'plugin_information' !== $action ) {
			return $res;
		}

		$plugin_slug = 'wp-ometrics'; // we are going to use it in many places in this function

		// do nothing if it is not our plugin
		if(!isset($args->slug) || $plugin_slug !== $args->slug ) {
			return $res;
		}

		// trying to get from cache first
		if( false == $remote = get_transient( 'update_wp-ometrics') ) {

			// info.json is the file with the actual plugin information on your server
			$remote = wp_remote_get( 'https://www.ometrics.com/asset/wp-ometrics.json', array(
				'timeout' => 10,
				'headers' => array(
					'Accept' => 'application/json'
				) )
			);

			if ( ! is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && ! empty( $remote['body'] ) ) {
				set_transient( 'update_wp-ometrics', $remote, 43200 ); // 12 hours cache
			}
		}

		if( ! is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && ! empty( $remote['body'] ) ) {

			$remote = json_decode( $remote['body'] );
			$res = new stdClass();

			$res->name = $remote->name;
			$res->slug = $plugin_slug;
			$res->version = $remote->version;
			$res->tested = $remote->tested;
			$res->requires = $remote->requires;
			$res->author = '<a href="'.$remote->author_homepage.'">'.$remote->author.'</a>';
			$res->author_profile = '';
			$res->download_link = $remote->download_url;
			$res->trunk = $remote->download_url;
			$res->requires_php = $remote->requires_php;
			$res->last_updated = $remote->last_updated;
			$res->sections = array(
				'description' => $remote->sections->description,
				'installation' => $remote->sections->installation,
				'changelog' => $remote->sections->changelog
				// you can add your custom sections (tabs) here
			);

			// in case you want the screenshots tab, use the following HTML format for its content:
			// <ol><li><a href="IMG_URL" target="_blank"><img src="IMG_URL" alt="CAPTION" /></a><p>CAPTION</p></li></ol>
			if( !empty( $remote->sections->screenshots ) ) {
				$res->sections['screenshots'] = $remote->sections->screenshots;
			}

			//$res->banners = array(
			//	'low' => 'https://YOUR_WEBSITE/banner-772x250.jpg',
			//	'high' => 'https://YOUR_WEBSITE/banner-1544x500.jpg'
			//);
			return $res;

		}

		return $res;

	}


	function wp_ometrics_push_update( $transient ){

		//Comment out these two lines during testing.
		//if ( empty($transient->checked ) ) {
		//	return $transient;
		//}

		// trying to get from cache first, to disable cache comment 10,20,21,22,24
		if( false == $remote = get_transient( 'update_wp-ometrics' ) ) {

			// info.json is the file with the actual plugin information on your server
			$remote = wp_remote_get( 'https://www.ometrics.com/asset/wp-ometrics.json', array(
				'timeout' => 10,
				'headers' => array(
					'Accept' => 'application/json'
				) )
			);

			if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && !empty( $remote['body'] ) ) {
				set_transient( 'update_wp-ometrics', $remote, 43200 ); // 12 hours cache
			}

		}

		if( $remote ) {

			$remote = json_decode( $remote['body'] );

			// your installed plugin version should be on the line below! You can obtain it dynamically of course
			if( $remote && version_compare( WP_OMETRICS_VERSION, $remote->version, '<' ) && version_compare($remote->requires, get_bloginfo('version'), '<' ) ) {
				$res = new stdClass();
				$res->slug = 'wp-ometrics';
				$res->plugin = 'wp-ometrics/wp-ometrics.php'; // it could be just YOUR_PLUGIN_SLUG.php if your plugin doesn't have its own directory
				$res->new_version = $remote->version;
				$res->tested = $remote->tested;
				$res->package = $remote->download_url;
				$transient->response[$res->plugin] = $res;
				//$transient->checked[$res->plugin] = $remote->version;
			}

		}
		return $transient;
	}

	function wp_ometrics_after_update( $upgrader_object, $options ) {
		if ( $options['action'] == 'update' && $options['type'] === 'plugin' )  {
			// just clean the cache when new plugin version is installed
			delete_transient( 'update_wp-ometrics' );
		}
	}

	function submitOmetricsSettings() {
		// Check nonce
		if (!check_ajax_referer( $this->plugin->name, $this->plugin->name.'_nonce' )) {
			$data = array (
				'error' => "Error saving data.  Please try again or <a href='https://www.ometrics.com/user/support'>contact Ometrics technical support</a>.",
			);

			wp_send_json($data);
		}
		else {
			// Save
			// $_REQUEST has already been slashed by wp_magic_quotes in wp-settings
			// so do nothing before saving
			update_option( 'ometrics_id', $_REQUEST['ometrics_id'] );
			update_option( 'ometrics_token', $_REQUEST['ometrics_token'] );
			update_option( $this->plugin->db_welcome_dismissed_key, 1 );

			$data = array (
				'success' => __( 'Settings Saved.', $this->plugin->name ),
			);

			wp_send_json($data);

		}

	}
    /**
     * Show relevant notices for the plugin
     */
     //?djb
    function dashboardNotices() {
        global $pagenow;

        if ( !get_option( $this->plugin->db_welcome_dismissed_key ) ) {
        	if ( ! ( $pagenow == 'options-general.php' && isset( $_GET['page'] ) && $_GET['page'] == 'wp-ometrics' ) ) {
	            $setting_page = admin_url( 'options-general.php?page=' . $this->plugin->name );
	            // load the notices view
                include_once( $this->plugin->folder . '/views/dashboard-notices.php' );
        	}
        }
    }

    /**
     * Dismiss the welcome notice for the plugin
     */
    function dismissDashboardNotices() {
    	check_ajax_referer( $this->plugin->name . '-nonce', 'nonce' );
        // user has dismissed the welcome notice
        update_option( $this->plugin->db_welcome_dismissed_key, 1 );
        exit;
    }

	/**
	* Register Settings
	*/
	function registerSettings() {
		// Embed the Script on our Plugin's Option Page Only
		if ( isset($_GET['page']) && $_GET['page'] == 'wp-ometrics' ) {
			wp_enqueue_script('jquery');
			wp_enqueue_script( 'jquery-form' );
		}
		$args = array(
      'type' => 'string',
      'sanitize_callback' => 'sanitize_text_field',
      'default' => NULL,
    );
		register_setting( $this->plugin->name, 'ometrics_id', $args );
		register_setting( $this->plugin->name, 'ometrics_token', $args );
	}

	/**
    * Register the plugin settings panel
    */
    function adminPanelsAndMetaBoxes() {
    	add_submenu_page( 'options-general.php', $this->plugin->displayName, $this->plugin->displayName, 'manage_options', $this->plugin->name, array( &$this, 'adminPanelConnect' ) );
	}

	/**
	* Output the Administration Panel
	* If no data saved, show Ometrics login page
	*		Login page - login to Ometrics and retrieve ometrics id and token
	* If data saved, show button to disconnect from Ometrics
	*		- Remove ometrics id and token
	* Save Ometrics ID and Token data from the Administration Panel into a WordPress option
	*/
	function adminPanelConnect() {
		// only admin user can access this page
		if ( !current_user_can( 'administrator' ) ) {
			echo '<p>' . __( 'Sorry, you are not allowed to access this page.', $this->plugin->name ) . '</p>';
			return;
		}

		// Save Settings
		if ( isset( $_REQUEST['submit'] ) ) {
			// Check nonce
			if ( !isset( $_REQUEST[$this->plugin->name.'_nonce'] ) ) {
				// Missing nonce
				$this->errorMessage = __( 'nonce field is missing. Settings NOT saved.', $this->plugin->name );
			} elseif ( !wp_verify_nonce( $_REQUEST[$this->plugin->name.'_nonce'], $this->plugin->name ) ) {
				// Invalid nonce
				$this->errorMessage = __( 'Invalid nonce specified. Settings NOT saved.', $this->plugin->name );
			} else {
				// Save
				// $_REQUEST has already been slashed by wp_magic_quotes in wp-settings
				// so do nothing before saving
				update_option( 'ometrics_id', $_REQUEST['ometrics_id'] );
				update_option( 'ometrics_token', $_REQUEST['ometrics_token'] );
				update_option( $this->plugin->db_welcome_dismissed_key, 1 );
				$this->message = __( 'Settings Saved.', $this->plugin->name );
			}
		}

		// Get latest settings
		$this->settings = array(
			'ometrics_id' => esc_html( wp_unslash( get_option( 'ometrics_id' ) ) ),
			'ometrics_token' => esc_html( wp_unslash( get_option( 'ometrics_token' ) ) ),
		);

		// Load Settings Form
		include_once( WP_PLUGIN_DIR . '/' . $this->plugin->name . '/views/settingsConnect.php' );
	}

	/**
	* Loads plugin textdomain
	*/
	function loadLanguageFiles() {
		load_plugin_textdomain( $this->plugin->name, false, $this->plugin->name . '/languages/' );
	}

	/**
	* Outputs script to the frontend header
	*/
	function frontendHeader() {
		$this->output( 'ometrics_id' );
		if ( class_exists( 'woocommerce' ) ) {
      //ajax remove item - don't process the cart update, wait for a page refresh
      if (!$_GET['removed_item']) {
        $this->cartUpdate();
      }
		}
	}

	public function checkoutComplete( $order_id ) {

		//Get order info
		$order = wc_get_order( $order_id );

		// bail out if not a valid instance
		if ( ! is_a( $order, 'WC_Order' ) ) {
				return;
		}

		$order_total    = $order->get_total() ? $order->get_total() : 0;
		$order_number   = $order->get_order_number();
		$items = $order->get_items();
		$num_items = $items ? count($items) : 0;

		// Output
		?>
		<!-- Ometrics - conversion tracking -->
		<script>
			window.ometricsAPI = window.ometricsAPI || [];

			window.ometricsAPI.push('o_trackEvent', 'purchase', {'revenue': <?php echo $order_total; ?>, num_items: <?php echo $num_items; ?> , transaction_id: '<?php echo $order_number; ?>' });
		</script>";
		<!-- Ometrics - end conversion tracking -->
		<?php
	}

	/**
	* Outputs the given setting, if conditions are met
	*
	* @param string $setting Setting Name
	* @return output
	*/
	function output( $setting ) {
		// Ignore admin, feed, robots or trackbacks
		if ( is_admin() || is_feed() || is_robots() || is_trackback() ) {
			return;
		}

		// Get meta
		$meta = get_option( $setting );
		if ( empty( $meta ) ) {
			return;
		}
		if ( trim( $meta ) == '' ) {
			return;
		}

		// Output
		?>
		<!-- Ometrics - place all of this script before </head> -->
		<object id="<?php echo wp_unslash( $meta );?>">
			<script type="text/javascript">var OMETRICS_WIDGET = "<?php echo wp_unslash( $meta );?>";</script>
			<script type="text/javascript">var HOST_URL = 'www.ometrics.com';</script>
			<script type="text/javascript" language="javascript" data-my="<?php echo wp_unslash( $meta );?>" src="https://www.ometrics.com/widget/widget.js"></script>
		</object>
		<!-- Ometrics - place all of this script before </head> -->
		<?php
	}

	function cartUpdate() {

		$upsellItem = '';
		$cart = WC()->cart->get_cart();
    $cartProcessed;
    if ($cart) {
      $cartProcessed = new stdClass;
      //preprocess cart - so, it's just the items, not shipping, tax, etc.
      $cartProcessed->subtotal_ex_tax = WC()->cart->subtotal_ex_tax;
      $cartProcessed->count = WC()->cart->get_cart_contents_count();
      $cartProcessed->items = [];
      //items
      foreach ( $cart as $cart_item_key => $cart_item ) {
        $item = new stdClass;
        $product = $cart_item['data'];
        if( version_compare( WC_VERSION, '3.0', '<' ) ){
          $product_id = $product->id; // Before version 3.0
        }
        else {
          $product_id = $product->get_id(); // For version 3 or more
        }
        $item->quantity = $cart_item['quantity'];
        $item->price = $product->get_price();
        $subtotal =  $price * $cart_item['quantity'];
        $item->url = $product->get_permalink( $cart_item );
        $item->image = wp_get_attachment_image_url( $product->get_image_id(), 'thumbnail' );
        $item->id = $product_id;
        $item->quantity = $cart_item['quantity'];
        $item->variant_id = $cart_item['variation_id'];
        $item->key = $cart_item['key'];
        $item->title = $product->get_name();

        $item->original_price = $item->price;
        $item->discounted_price = $item->price;
        $item->line_price = $subtotal;
        $item->original_line_price = $subtotal;
        $item->total_discount = 0;
        $item->sku = $product->get_sku();
        $item->grams = 0;
        $item->vendor = '';
        $item->taxable = 0;
        $item->product_id = $cart_item['product_id'];
        $item->final_price = $price;
        $item->final_line_price = $subtotal;
        $item->handle = '';
        $item->requires_shipping = 0;
        $cartProcessed->items[] = $item;
      }
    }

		$savedCart = WC()->session->get( 'ometricsCart' );

		if (!$cartProcessed && $savedCart) {
			//no cart, but there was a saved cart
			//javascript to remove abandoned cart info from local storage
			WC()->session->set( 'ometricsCart' , null );
			// Output
			?>
      <script type="text/javascript">
        if (window.timerOchatbotCart)  clearInterval(window.timerOchatbotCart);
        window.timerOchatbotCart = setInterval(function() {
          if (typeof chatId !== 'undefined') {
            clearInterval(window.timerOchatbotCart);
            //clear cart
            window.localStorage.removeItem('ometricsCart'+chatId);
    				window.localStorage.removeItem('ometricsCartDate'+chatId);
          }
        }, 100);

			</script>
			<?php
		}
		elseif ($cartProcessed != $savedCart) {
			// cart has change from saved cart
			//javascript to update abandoned cart in local storage
			WC()->session->set( 'ometricsCart' , $cartProcessed );
      //check if an item has been added
      foreach ( $cartProcessed->items as $cart_item_key => $cart_item ) {
        //check if this item is in the saved cart
        $itemFound = false;
        if (isset($savedCart->items)) {
          foreach ( $savedCart->items as $saved_cart_item_key => $saved_cart_item ) {
            //check if this item is in the saved cart
            if ($saved_cart_item->id == $cart_item->id) {
              $itemFound = true;
              //write item to js for upsell
              break;
            }

          }
        }
        if (!$itemFound) {
          //didn't find this item - so, upsell it and then exit out of loop
          $upsellItem = $cart_item->id;
          break;
        }
      }
			// Output
			?>
			<script type="text/javascript">
      	try {

					var cart = {};
		      cart.total_price = <?php echo $cartProcessed->subtotal_ex_tax ? $cartProcessed->subtotal_ex_tax : 0; ?>;
		      cart.item_count = <?php echo $cartProcessed->count ? $cartProcessed->count : 0; ?>;
		      cart.items = [];
		      //items
					<?php
					foreach ( $cartProcessed->items as $cart_item_key => $cart_item ) {
					?>
		        var newItem = {};
		        newItem.id = <?php echo $cart_item->id; ?>;
		        newItem.quantity = <?php echo $cart_item->quantity; ?>;
		        newItem.variant_id = <?php echo $cart_item->variant_id; ?>;
		        newItem.key = '<?php echo $cart_item->key; ?>';
		        newItem.title = encodeURIComponent('<?php echo $cart_item->title; ?>');
		        newItem.price = <?php echo $cart_item->price; ?>;
		        newItem.original_price = <?php echo $cart_item->price; ?>;
		        newItem.discounted_price = <?php echo $cart_item->price; ?>;
		        newItem.line_price = <?php echo $cart_item->line_price; ?>;
		        newItem.original_line_price = <?php echo $cart_item->original_line_price; ?>;
		        newItem.total_discount = 0;
		        newItem.sku = '<?php echo $cart_item->sku; ?>';
		        newItem.grams = 0;
		        newItem.vendor = '';
		        newItem.taxable = 0;
		        newItem.product_id = <?php echo $cart_item->product_id; ?>;
		        newItem.final_price = <?php echo $cart_item->price; ?>;
		        newItem.final_line_price = <?php echo $cart_item->final_line_price; ?>;
		        newItem.url = '<?php echo $cart_item->url; ?>';
		        newItem.image = '<?php echo $cart_item->image; ?>';
		        newItem.handle = '';
		        newItem.requires_shipping = 0;

		        cart.items.push(newItem);
		    <?php
			 	}
			 	?>
          var wooUpsellItem = '<?php echo $upsellItem; ?>';
					if (window.timerOchatbotCart)  clearInterval(window.timerOchatbotCart);
					window.timerOchatbotCart = setInterval(function() {
						if (typeof chatId !== 'undefined') {
							clearInterval(window.timerOchatbotCart);
							//create new cart in local storage for this ochatbot
							window.localStorage.setItem('ometricsCart'+chatId, JSON.stringify(cart));
							d = new Date();
							window.localStorage.setItem('ometricsCartDate'+chatId, d.getTime());
              //
						}
					}, 100);
				}
				catch (e) {
					console.log("error adding to ometrics cart: ", e.message);
				}
			</script>
			<?php
		}
	}

	/**
   * Check if a given request has access to get items
   *
   * @param WP_REST_Request $request Full data about the request.
   * @return WP_Error|bool
   */
  public function ometricsPermissionCheck( WP_REST_Request $request ) {
		//check token and id match
		if(!$request->get_param('ometrics_token') || !$request->get_param('ometrics_id') || $request->get_param('ometrics_token') !== get_option( 'ometrics_token' ) || $request->get_param('ometrics_id') !== get_option( 'ometrics_id' )) {
		//if(!$request->get_param('ometrics_token') || $request->get_param('ometrics_token') !== get_option( 'ometrics_id' )) {
			return false;
		}

		//authenticated ochatbot api call - so, allow
		return true;
  }

	/**
 * Get the order status by id
 *
 * @param array $data Options for the function.
 * @return string|null order status data
 */
	public function ochatbotOrderStatus(WP_REST_Request $request) {
		$orderNumber = $request['order_id'];
		//handle sequential order number plugin -
		// how to handle other plugins?
		if ( function_exists( 'wc_sequential_order_numbers' ) ) {
			// Try to convert number for Sequential Order Number.
			$orderId = wc_sequential_order_numbers()->find_order_by_order_number( $orderNumber );

		} elseif ( function_exists( 'wc_seq_order_number_pro' ) ) {
			// Try to convert number for Sequential Order Number Pro.
			$orderId = wc_seq_order_number_pro()->find_order_by_order_number( $orderNumber );

		} else {
			// Default to not converting order number.
			$orderId = $orderNumber;
		}

		if ( 0 === $orderId ) {
			$orderId = $orderNumber;
		}

		//lookup the order
		$order = wc_get_order( $orderId );

		if ( false === $order || ! is_object( $order ) ) {
			//order not found -
			//return the not found response
			//exit;
			return new WP_Error( 'no_order', 'Invalid Order', array( 'status' => 404 ) );
		}

		// Get real order ID from order object.
		$order_id = version_compare( WC_VERSION, '3.0.0', '<' ) ? $order->id : $order->get_id();
		if ( empty( $order_id ) ) {
			//order not found -
			//return the not found response
			return new WP_Error( 'no_order', 'Invalid Order', array( 'status' => 404 ) );

		}

		//Shipment Tracking Plugins
		$trackingInfo = [];

		// https://docs.woocommerce.com/document/shipment-tracking/
		// if custom meta exists: _wc_shipment_tracking_items
		// or, use api: GET /wp-json/wc-shipment-tracking/v3/orders/<order_id>/shipment-trackings/
		$shipment_data = $order->get_meta('_wc_shipment_tracking_items');
		if ($shipment_data) {
			$trackingInfo = $shipment_data;
		}

		// https://github.com/helloteababy/woocommerce-shipment-tracking/blob/master/woocommerce-advanced-shipment-tracking.php
		// function exists - wc_advanced_shipment_tracking
		// $tracking_items = $order->get_meta( '_wc_shipment_tracking_items', true );

		//https://www.pluginhive.com/woocommerce-order-tracking-plugin-free/
		// class exists - WooCommerce_Shipment_Tracking
		// $shipment_result_array 	= get_post_meta( $order_id , 'wf_wc_shipment_result', true );

		// https://wordpress.org/plugins/woo-advanced-shipment-tracking/
		// function exists - wc_advanced_shipment_tracking
		// class exists - zorem_woocommerce_advanced_shipment_tracking
		//$tracking_items = $order->get_meta( '_wc_shipment_tracking_items', true );
		//$shipment_status = get_post_meta( $order_id, "shipment_status", true);
		// see - class-wc-advanced-shipment-tracking-api-call.php


		//get the order status and notes (and, optionally, the tracking number)
		$data = array('order_number' => $order->get_order_number(),
			'order_date' => wc_format_datetime( $order->get_date_created() ),
			'order_status' => wc_get_order_status_name( $order->get_status() ),
			'order_notes' => $order->get_customer_order_notes(),
			'tracking_info' => $trackingInfo,
		);

		// Create the response object
		$response = new WP_REST_Response( $data );

		return $response;
	}

	public function allowRestCallForAuthenticatedPlugin($permission,  $context,  $object_id,  $type) {
		//allow reading products for authenticated call
		//check url
		global $wp;

		$url = home_url( $wp->request );
		if (!preg_match('/wp-ometrics\/'.$this->ometrics_api_version.'\/products|wp-oometrics\/'.$this->ometrics_api_version.'\/products\/[\d]+\/variations/', $url)) {
			return $permission;
		}
		//check token and id match
		if(!$_GET['ometrics_token'] || !$_GET['ometrics_id'] || $_GET['ometrics_token'] !== get_option( 'ometrics_token' ) || $_GET['ometrics_id'] !== get_option( 'ometrics_id' ) ) {
		//if(!$_GET['ometrics_token'] || $_GET['ometrics_token'] !== get_option( 'ometrics_id' )) {
			return $permission;
		}

		//authenticated ochatbot api call - so, allow
		return true;
	}

	public function ochatbotProductList(WP_REST_Request $request) {
		//permissions have already been checked in the permission callback
		// so, set up filter to allow the rest call
		//add_filter( 'woocommerce_rest_check_permissions', array( $this, 'allowRestCallForAuthenticatedPlugin' ), 90, 4 );

		//call the existing product list api with the posted parameters
		$productRequest = new WP_REST_Request( 'GET', '/wc/'.$this->wp_api_version.'/products' );

		//remove the filter to allow the rest call
		//remove_filter( 'woocommerce_rest_check_permissions', array( $this, 'allowRestCallForAuthenticatedPlugin' ), 90, 4 );


		$productRequest->set_query_params( $request->get_query_params() );

		return rest_do_request( $productRequest );
		//$server = rest_get_server();
		//$data = $server->response_to_data( $response, false );
	}

	public function ochatbotProductVariation(WP_REST_Request $request) {
		$productId = $request['product_id'];
		//permissions have already been checked in the permission callback
		// so, set up filter to allow the rest call
		//add_filter( 'woocommerce_rest_check_permissions', array( $this, 'allowRestCallForAuthenticatedPlugin' ), 90, 4 );

		//call the existing product list api with the posted parameters
		$productRequest = new WP_REST_Request( 'GET', '/wc/'.$this->wp_api_version.'/products/'.$productId.'/variations' );

		//remove the filter to allow the rest call
		//remove_filter( 'woocommerce_rest_check_permissions', array( $this, 'allowRestCallForAuthenticatedPlugin' ), 90, 4 );


		$productRequest->set_query_params( $request->get_query_params() );

		return rest_do_request( $productRequest );
		//$server = rest_get_server();
		//$data = $server->response_to_data( $response, false );
	}

}

$ometrics = new WPOmetrics();
